@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2003 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -14,7 +14,7 @@
 #endif
 #include	"imapwrite.h"
 
-static const char rcsid[]="$Id: capability.c,v 1.6 2003/02/13 03:19:45 mrsam Exp $";
+static const char rcsid[]="$Id: capability.c,v 1.11 2006/02/24 02:15:07 mrsam Exp $";
 
 int have_starttls()
 {
@@ -38,6 +38,24 @@ const char *p=getenv("IMAP_TLS_REQUIRED");
 	return (0);
 }
 
+int keywords()
+{
+	const char *p=getenv("IMAP_KEYWORDS");
+
+	if (p && atoi(p))       return (1);
+	return (0);
+}
+
+int magictrash()
+{
+	const char *p;
+
+	p=getenv("IMAP_MOVE_EXPUNGE_TO_TRASH");
+
+	if (p && atoi(p))
+		return 1;
+	return 0;
+}
 
 void imapcapability()
 {
@@ -51,6 +69,22 @@ void imapcapability()
 	else
 		writes("IMAP4rev1");
 
+#if SMAP
+	p=getenv("SMAP_CAPABILITY");
+
+	if (p && *p)
+	{
+		writes(" ");
+		writes(p);
+
+		if (keywords())
+			writes(" KEYWORDS");
+	}
+#endif
+
+	if ((p=getenv("IMAP_ACL")) && atoi(p))
+		writes(" ACL ACL2=UNION");
+
 	if (have_starttls())
 	{
 		writes(" STARTTLS");
@@ -65,4 +99,7 @@ void imapcapability()
 		writes(" XCOURIEROUTBOX=INBOX");
 		writes(p);
 	}
+
+	if (magictrash())
+		writes(" XMAGICTRASH");
 }
