@@ -1,8 +1,8 @@
 /*
-** Copyright 1998 - 2004 Double Precision, Inc.
+** Copyright 1998 - 2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.37 2005/03/01 03:11:24 mrsam Exp $
+** $Id: pop3dserver.c,v 1.38 2005/10/01 03:23:44 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -106,6 +106,9 @@ static int enomem_1msg;
 static unsigned long top_count=0;
 static unsigned long retr_count=0;
 
+static unsigned long bytes_sent_count=0;
+static unsigned long bytes_received_count=0;
+
 static unsigned long uidv=0;
 static int convert_v0=0;
 
@@ -558,6 +561,18 @@ static void cleanup()
 	return;
 }
 
+static void printed(int cnt)
+{
+	if (cnt > 0)
+		bytes_sent_count += cnt;
+}
+
+static void printchar(int ch)
+{
+	putchar(ch);
+	printed(1);
+}
+
 /* POP3 STAT */
 
 static void do_stat()
@@ -574,7 +589,7 @@ char	buf[NUMBUFSIZE];
 		++j;
 	}
 
-	printf("+OK %u %s\r\n", j, libmail_str_off_t(n, buf));
+	printed(printf("+OK %u %s\r\n", j, libmail_str_off_t(n, buf)));
 	fflush(stdout);
 }
 
@@ -585,7 +600,7 @@ unsigned i;
 	if (!p || (i=atoi(p)) > msglist_cnt || i == 0 ||
 		msglist_a[i-1]->isdeleted)
 	{
-		printf("-ERR Invalid message number.\r\n");
+		printed(printf("-ERR Invalid message number.\r\n"));
 		fflush(stdout);
 		return (0);
 	}
@@ -603,20 +618,21 @@ char	buf[NUMBUFSIZE];
 	{
 		if ((i=getmsgnum(msgnum)) != 0)
 		{
-			printf("+OK %u %s\r\n", i,
-				libmail_str_off_t(msglist_a[i-1]->size, buf));
+			printed(printf("+OK %u %s\r\n", i,
+				       libmail_str_off_t(msglist_a[i-1]->size,
+							 buf)));
 			fflush(stdout);
 		}
 		return;
 	}
 
-	printf("+OK POP3 clients that break here, they violate STD53.\r\n");
+	printed(printf("+OK POP3 clients that break here, they violate STD53.\r\n"));
 	for (i=0; i<msglist_cnt; i++)
 	{
 		if (msglist_a[i]->isdeleted)	continue;
-		printf("%u %s\r\n", i+1, libmail_str_off_t(msglist_a[i]->size, buf));
+		printed(printf("%u %s\r\n", i+1, libmail_str_off_t(msglist_a[i]->size, buf)));
 	}
-	printf(".\r\n");
+	printed(printf(".\r\n"));
 	fflush(stdout);
 }
 
@@ -633,12 +649,12 @@ unsigned long *cntr;
 
 	if (!f)
 	{
-		printf("-ERR Can't open the message file - it's gone!\r\n");
+		printed(printf("-ERR Can't open the message file - it's gone!\r\n"));
 		fflush(stdout);
 		return;
 	}
-	printf( (lptr ? "+OK headers follow.\r\n":"+OK %s octets follow.\r\n"),
-		libmail_str_off_t(msglist_a[i]->size, buf));
+	printed(printf( (lptr ? "+OK headers follow.\r\n":"+OK %s octets follow.\r\n"),
+			libmail_str_off_t(msglist_a[i]->size, buf)));
 
 	cntr= &retr_count;
 	if (lptr)
@@ -658,10 +674,10 @@ unsigned long *cntr;
 			}
 
 			if (c == '.')
-				putchar('.');
+				printchar('.');
 		}
-		if (c == '\n')	putchar('\r');
-		putchar(c);
+		if (c == '\n')	printchar('\r');
+		printchar(c);
 		++*cntr;
 	}
 	if (ferror(f)) {
@@ -671,8 +687,8 @@ unsigned long *cntr;
 		acctout("INFO: I/O error disconnect");
 		exit(1);
 	}
-	if (lastc != '\n')	printf("\r\n");
-	printf(".\r\n");
+	if (lastc != '\n')	printed(printf("\r\n"));
+	printed(printf(".\r\n"));
 	fflush(stdout);
 	fclose(f);
 	if (lptr)	return;
@@ -715,10 +731,11 @@ static void print_uidl(unsigned i)
 		char ino_buf[NUMBUFSIZE];
 		char mtime_buf[NUMBUFSIZE];
 
-		printf("ENOMEM-%s-%s-%s\r\n",
-		       libmail_strh_time_t(enomem_stat.st_mtime, mtime_buf),
-		       libmail_strh_dev_t(enomem_stat.st_dev, dev_buf),
-		       libmail_strh_ino_t(enomem_stat.st_ino, ino_buf));
+		printed(printf("ENOMEM-%s-%s-%s\r\n",
+			       libmail_strh_time_t(enomem_stat.st_mtime, mtime_buf),
+			       libmail_strh_dev_t(enomem_stat.st_dev, dev_buf),
+			       libmail_strh_ino_t(enomem_stat.st_ino, ino_buf))
+			);
 		return;
 	}
 
@@ -726,9 +743,9 @@ static void print_uidl(unsigned i)
 	{
 		/* VERSION 1 and VERSION 2 UIDL */
 
-		printf((msglist_a[i]->uid.uidv ?
-			"UID%lu-%lu\r\n":"UID%lu\r\n"),
-		       msglist_a[i]->uid.n, msglist_a[i]->uid.uidv);
+		printed(printf((msglist_a[i]->uid.uidv ?
+				"UID%lu-%lu\r\n":"UID%lu\r\n"),
+			       msglist_a[i]->uid.n, msglist_a[i]->uid.uidv));
 		return;
 	}
 
@@ -740,12 +757,12 @@ static void print_uidl(unsigned i)
 	{
 		if (*p < 0x21 || *p > 0x7E || *p == '\'' || *p == '"' ||
 			*p == '+')
-			printf("+%02X", (int)(unsigned char)*p);
+			printed(printf("+%02X", (int)(unsigned char)*p));
 		else
-			putchar(*p);
+			printchar(*p);
 		++p;
 	}
-	printf("\r\n");
+	printed(printf("\r\n"));
 }
 
 static void do_uidl(const char *msgnum)
@@ -756,20 +773,20 @@ unsigned i;
 	{
 		if ((i=getmsgnum(msgnum)) != 0)
 		{
-			printf("+OK %u ", i);
+			printed(printf("+OK %u ", i));
 			print_uidl(i-1);
 			fflush(stdout);
 		}
 		return;
 	}
-	printf("+OK\r\n");
+	printed(printf("+OK\r\n"));
 	for (i=0; i<msglist_cnt; i++)
 	{
 		if (msglist_a[i]->isdeleted)	continue;
-		printf("%u ", i+1);
+		printed(printf("%u ", i+1));
 		print_uidl(i);
 	}
-	printf(".\r\n");
+	printed(printf(".\r\n"));
 	fflush(stdout);
 }
 
@@ -781,10 +798,14 @@ static void acctout(const char *disc)
 	static const char msg5[]=", retr=";
 	static const char msg6[]=", time=";
 	static const char msg7[]=", stls=1";
+	static const char msgAR[]=", rcvd=";
+	static const char msgAS[]=", sent=";
 
 	char num1[NUMBUFSIZE];
 	char num2[NUMBUFSIZE];
 	char num3[NUMBUFSIZE];
+	char numAR[NUMBUFSIZE];
+	char numAS[NUMBUFSIZE];
 
 	char *p;
 	const char *q;
@@ -792,9 +813,12 @@ static void acctout(const char *disc)
 	libmail_str_size_t(top_count, num1);
 	libmail_str_size_t(retr_count, num2);
 	libmail_str_time_t(time(NULL)-start_time, num3);
+	libmail_str_size_t(bytes_received_count, numAR);
+	libmail_str_size_t(bytes_sent_count, numAS);
 
 	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(disc)+
-		 strlen(num1)+strlen(num2)+strlen(num3)+200);	/* Should be enough */
+		 strlen(num1)+strlen(num2)+strlen(num3)+
+		 strlen(numAR)+strlen(numAS)+200);	/* Should be enough */
 
 	strcpy(p, disc);
 	strcat(p, msg2);
@@ -805,6 +829,10 @@ static void acctout(const char *disc)
 	strcat(p, num1);
 	strcat(p, msg5);
 	strcat(p, num2);
+	strcat(p, msgAR);
+	strcat(p, numAR);
+	strcat(p, msgAS);
+	strcat(p, numAS);
 	strcat(p, msg6);
 	strcat(p, num3);
 
@@ -834,6 +862,8 @@ int	c;
 	signal(SIGALRM, bye);
 	while (alarm(300), fgets(buf, sizeof(buf), stdin))
 	{
+		bytes_received_count += strlen(buf);
+
 		alarm(0);
 		if ((p=strchr(buf, '\n')) != 0)
 			*p=0;
@@ -845,7 +875,7 @@ int	c;
 		mkupper(p);
 		if (strcmp(p, "QUIT") == 0)
 		{
-			printf("+OK Bye-bye.\r\n");
+			printed(printf("+OK Bye-bye.\r\n"));
 			fflush(stdout);
 			cleanup();
 			acctout("INFO: LOGOUT");
@@ -889,14 +919,14 @@ int	c;
 				continue;
 
 			msglist_a[i-1]->isdeleted=1;
-			printf("+OK Deleted.\r\n");
+			printed(printf("+OK Deleted.\r\n"));
 			fflush(stdout);
 			continue;
 		}
 
 		if (strcmp(p, "NOOP") == 0)
 		{
-			printf("+OK Yup.\r\n");
+			printed(printf("+OK Yup.\r\n"));
 			fflush(stdout);
 			continue;
 		}
@@ -907,7 +937,7 @@ int	c;
 
 			for (i=0; i<msglist_cnt; i++)
 				msglist_a[i]->isdeleted=0;
-			printf("+OK Resurrected.\r\n");
+			printed(printf("+OK Resurrected.\r\n"));
 			fflush(stdout);
 			continue;
 		}
@@ -936,7 +966,7 @@ int	c;
 		}
 
 error:
-		printf("-ERR Invalid command.\r\n");
+		printed(printf("-ERR Invalid command.\r\n"));
 		fflush(stdout);
 	}
 	acctout("INFO: DISCONNECTED");
@@ -996,7 +1026,7 @@ char	*p;
 		{
 			fprintf(stderr, "INFO: LOCKED, user=%s, ip=[%s]\n",
 				authaddr, remoteip);
-			printf("-ERR Your account is temporarily unavailable (+t bit set on home directory).\r\n");
+			printed(printf("-ERR Your account is temporarily unavailable (+t bit set on home directory).\r\n"));
 			exit(0);
 		}
 	}
@@ -1012,7 +1042,7 @@ char	*p;
 	if (chdir(p))
 	{
 		fprintf(stderr, "chdir %s: %s\n", p, strerror(errno));
-		printf("-ERR chdir %s failed\n", p);
+		printed(printf("-ERR chdir %s failed\n", p));
 		fflush(stdout);
 		exit(1);
 	}
@@ -1021,7 +1051,7 @@ char	*p;
 
 	if (auth_getoptionenvint("disablepop3"))
 	{
-		printf("-ERR POP3 access disabled for this account.\r\n");
+		printed(printf("-ERR POP3 access disabled for this account.\r\n"));
 		fflush(stdout);
 		exit(1);
 	}
@@ -1038,11 +1068,11 @@ char	*p;
 	maildir_getnew(".", INBOX, NULL, NULL);
 	if (scancur())
 	{
-		printf("-ERR Maildir invalid (no 'cur' directory)\r\n");
+		printed(printf("-ERR Maildir invalid (no 'cur' directory)\r\n"));
 		return (0);
 	}
 	sortmsgs();
-	printf("+OK logged in.\r\n");
+	printed(printf("+OK logged in.\r\n"));
 	fflush(stdout);
 	loop();
 	return (0);
