@@ -28,15 +28,18 @@
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.25 2003/02/10 04:24:51 mrsam Exp $";
+static const char rcsid[]="$Id: search.c,v 1.32 2005/08/11 23:42:33 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
-extern struct imapscaninfo current_mailbox_info;
+extern struct imapscaninfo current_maildir_info;
 extern char *current_mailbox;
 
 extern int get_flagname(const char *, struct imapflags *);
 extern void get_message_flags( struct imapscanmessageinfo *,
 	char *, struct imapflags *);
+extern int valid_keyword(const char *kw);
+
+static void fill_search_preparse(struct searchinfo *);
 
 static void fill_search_veryquick(struct searchinfo *,
 	unsigned long, struct imapflags *);
@@ -84,9 +87,27 @@ static void search_callback(struct searchinfo *si, struct searchinfo *sihead,
 	int isuid, unsigned long i, void *dummy)
 {
 	writes(" ");
-	writen(isuid ? current_mailbox_info.msgs[i].uid:i+1);
+	writen(isuid ? current_maildir_info.msgs[i].uid:i+1);
 }
 
+static void search_oneatatime(struct searchinfo *si,
+			      unsigned long i,
+			      struct searchinfo *sihead,
+			      const struct unicode_info *mycharset, int isuid,
+			      void (*callback_func)(struct searchinfo *,
+						    struct searchinfo *, int,
+						    unsigned long, void *),
+			      void *voidarg);
+
+static void search_byKeyword(struct searchinfo *tree,
+			   struct searchinfo *keyword,
+			   struct searchinfo *sihead,
+			   const struct unicode_info *mycharset, int isuid,
+			   void (*callback_func)(struct searchinfo *,
+						 struct searchinfo *, int,
+						 unsigned long, void *),
+			   void *voidarg);
+
 void search_internal(struct searchinfo *si, struct searchinfo *sihead,
 		     const struct unicode_info *mycharset, int isuid,
 		     void (*callback_func)(struct searchinfo *,
@@ -94,7 +115,65 @@ void search_internal(struct searchinfo *si, struct searchinfo *sihead,
 					   unsigned long, void *),
 		     void *voidarg)
 {
-unsigned long i;
+	unsigned long i;
+	struct	searchinfo *p;
+
+	for (p=sihead; p; p=p->next)
+		fill_search_preparse(p);
+
+	/* Shortcuts for keyword-based searches */
+
+	if (si->type == search_msgkeyword && si->bs == NULL && si->ke)
+		search_byKeyword(NULL, si, sihead, mycharset, isuid,
+			       callback_func, voidarg);
+	else if (si->type == search_and &&
+		 si->a->type == search_msgkeyword && si->a->bs == NULL
+		 && si->a->ke)
+		search_byKeyword(si->b, si->a, sihead, mycharset, isuid,
+			       callback_func, voidarg);
+	else for (i=0; i<current_maildir_info.nmessages; i++)
+		search_oneatatime(si, i, sihead, mycharset, isuid,
+				  callback_func, voidarg);
+}
+
+static void search_byKeyword(struct searchinfo *tree,
+			     struct searchinfo *keyword,
+			     struct searchinfo *sihead,
+			     const struct unicode_info *mycharset, int isuid,
+			     void (*callback_func)(struct searchinfo *,
+						   struct searchinfo *, int,
+						   unsigned long, void *),
+			     void *voidarg)
+{
+	struct libmail_kwMessageEntry *kme;
+
+	for (kme=keyword->ke->firstMsg; kme; kme=kme->keywordNext)
+	{
+		unsigned long n=kme->libmail_kwMessagePtr->u.userNum;
+		if (!tree)
+		{
+			(*callback_func)(keyword, keyword, isuid, n, voidarg);
+			continue;
+		}
+
+		search_oneatatime(tree, n, sihead, mycharset, isuid,
+				  callback_func, voidarg);
+	}
+}
+
+/*
+** Evaluate the search tree for a given message.
+*/
+
+static void search_oneatatime(struct searchinfo *si,
+			      unsigned long i,
+			      struct searchinfo *sihead,
+			      const struct unicode_info *mycharset, int isuid,
+			      void (*callback_func)(struct searchinfo *,
+						    struct searchinfo *, int,
+						    unsigned long, void *),
+			      void *voidarg)
+{
 struct	searchinfo *p;
 struct	imapflags	flags;
 int	fd;
@@ -102,14 +181,13 @@ FILE	*fp;
 struct	stat	stat_buf;
 int	rc;
 
-	for (i=0; i<current_mailbox_info.nmessages; i++)
 	{
 		for (p=sihead; p; p=p->next)
 			p->value= -1;	/* Search result unknown */
 
 		/* First, see if non-content search will be sufficient */
 
-		get_message_flags(current_mailbox_info.msgs+i, 0, &flags);
+		get_message_flags(current_maildir_info.msgs+i, 0, &flags);
 
 		for (p=sihead; p; p=p->next)
 			fill_search_veryquick(p, i, &flags);
@@ -119,12 +197,12 @@ int	rc;
 			if (rc > 0)
 				(*callback_func)(si, sihead, isuid, i,
 						 voidarg);
-			continue;
+			return;
 		}
 
 		fd=imapscan_openfile(current_mailbox,
-			&current_mailbox_info, i);
-		if (fd < 0)	continue;
+			&current_maildir_info, i);
+		if (fd < 0)	return;
 
 		if ((fp=fdopen(fd, "r")) == 0)
 			write_error_exit(0);
@@ -132,7 +210,7 @@ int	rc;
 		if (fstat(fileno(fp), &stat_buf))
 		{
 			fclose(fp);
-			continue;
+			return;
 		}
 
 		/* First, see if non-content search will be sufficient */
@@ -147,7 +225,7 @@ int	rc;
                         struct        rfc2045 *rfcp=rfc2045header_fromfp(fp); 
 
 			fill_search_header(sihead, mycharset, rfcp, fp,
-				current_mailbox_info.msgs+i);
+				current_maildir_info.msgs+i);
 			rc=search_evaluate(si);
                         rfc2045_free(rfcp); 
 
@@ -156,8 +234,18 @@ int	rc;
 				/* Ok, search message contents */
                                 struct        rfc2045 *rfcp=rfc2045_fromfp(fp);
 
-				fill_search_body(si, mycharset, rfcp, fp,
-					current_mailbox_info.msgs+i);
+				fill_search_body(sihead, mycharset, rfcp, fp,
+					current_maildir_info.msgs+i);
+
+				/*
+				** If there are still UNKNOWN nodes, change
+				** them to fail.
+				*/
+
+				for (p=sihead; p; p=p->next)
+					if (p->value < 0)
+						p->value=0;
+
 				rc=search_evaluate(si);
                                 rfc2045_free(rfcp);
 			}
@@ -278,32 +366,89 @@ char	buf[200];
 	return (my_strdup(buf));
 }
 
+static void fill_search_preparse(struct searchinfo *p)
+{
+	switch (p->type) {
+	case search_msgflag:
+		{
+			struct imapflags flags;
+
+			memset(&flags, 0, sizeof(flags));
+			p->ke=NULL;
+
+			if (get_flagname(p->as, &flags) == 0)
+			{
+				p->bs=malloc(sizeof(flags));
+
+				if (!p->bs)
+					write_error_exit(0);
+
+				memcpy(p->bs, &flags, sizeof(flags));
+			}
+		}
+		break;
+
+	case search_msgkeyword:
+		p->ke=NULL;
+		if (valid_keyword(p->as))
+			p->ke=libmail_kweFind(current_maildir_info
+					       .keywordList,
+					       p->as, 0);
+		break;
+	default:
+		break;
+	}
+}
+
 /* Evaluate non-content search nodes */
 
 static void fill_search_veryquick(struct searchinfo *p,
 	unsigned long msgnum, struct imapflags *flags)
 {
 	switch (p->type) {
-	case search_keyword:
+	case search_msgflag:
 		{
-		struct imapflags f;
+			struct imapflags *f=(struct imapflags *)p->bs;
 
-			memset(&f, 0, sizeof(f));
 			p->value=0;
-			get_flagname(p->as, &f);
 			if (strcmp(p->as, "\\RECENT") == 0 &&
-				current_mailbox_info.msgs[msgnum].recentflag)
-				p->value=1;
-			if (f.seen && flags->seen)
-				p->value=1;
-			if (f.answered && flags->answered)
-				p->value=1;
-			if (f.deleted && flags->deleted)
-				p->value=1;
-			if (f.flagged && flags->flagged)
-				p->value=1;
-			if (f.drafts && flags->drafts)
+				current_maildir_info.msgs[msgnum].recentflag)
 				p->value=1;
+
+			if (f)
+			{
+				if (f->seen && flags->seen)
+					p->value=1;
+				if (f->answered && flags->answered)
+					p->value=1;
+				if (f->deleted && flags->deleted)
+					p->value=1;
+				if (f->flagged && flags->flagged)
+					p->value=1;
+				if (f->drafts && flags->drafts)
+					p->value=1;
+			}
+			break;
+		}
+
+	case search_msgkeyword:
+		p->value=0;
+		if (p->ke)
+		{
+			struct libmail_kwMessage *km=
+				current_maildir_info.msgs[msgnum]
+				.keywordMsg;
+			struct libmail_kwMessageEntry *kme;
+
+			for (kme=km ? km->firstEntry:NULL;
+			     kme; kme=kme->next)
+				if (strcasecmp(keywordName(kme->
+							   libmail_keywordEntryPtr),
+					       keywordName(p->ke))==0)
+				{
+					p->value=1;
+					break;
+				}
 		}
 		break;
 	case search_messageset:
@@ -316,8 +461,10 @@ static void fill_search_veryquick(struct searchinfo *p,
 		p->value=1;
 		break;
 	case search_uid:
-		if (is_in_set(p->as, current_mailbox_info.msgs[msgnum].uid))
+		if (is_in_set(p->as, current_maildir_info.msgs[msgnum].uid))
 			p->value=1;
+		else
+			p->value=0;
 		break;
 	case search_reverse:
 		p->value=1;
@@ -935,7 +1082,6 @@ static int search_body_func(const char *ptr, size_t cnt, void *voidptr)
 
 	struct searchinfo *si, *sihead=(struct searchinfo *)slb->si;
 	size_t	i, j;
-	int rc=1;
 
 	for (i=0; i<cnt; i++)
 	{
@@ -957,8 +1103,8 @@ static int search_body_func(const char *ptr, size_t cnt, void *voidptr)
 		{
 			char *q;
 
-			p=unicode_convert(slb->buf, uc_info,
-					  uc_info->search_chset);
+			p=unicode_xconvert(slb->buf, uc_info,
+					   uc_info->search_chset);
 
 			if (p)
 			{
@@ -989,8 +1135,6 @@ static int search_body_func(const char *ptr, size_t cnt, void *voidptr)
 
 				if (maildir_search_found(&si->sei))
 					si->value=1;
-				else
-					rc=0;
 			}
 		}
 		free(p);
